from typing import Type, TypeVar, overload
from uuid import UUID

from fastapi import HTTPException
from pydantic import BaseModel
from sqlalchemy.orm import Session
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.future import select
import asyncio
from .models.base import Base

T = TypeVar("T", bound=Base)


@overload
async def get_object_or_404(
    db_class: Type[T],
    id: UUID | str,
    db: AsyncSession,
    expunge: bool = False,
    lookup_column: str = "id",
) -> T:
    pass


@overload
def get_object_or_404(
    db_class: Type[T], id: UUID | str, db: Session, expunge: bool = False, lookup_column: str = "id"
) -> T:
    pass


def get_object_or_404(
    db_class: Type[T],
    id: UUID | str,
    db: Session | AsyncSession,
    expunge: bool = False,
    lookup_column: str = "id",
) -> T:

    async def _get_async_object() -> T:
        query = select(db_class).filter(getattr(db_class, lookup_column) == id)
        result = await db.execute(query)
        obj = result.scalar_one_or_none()
        if obj is None:
            raise HTTPException(status_code=404, detail="The object does not exist.")  # type: ignore
        if expunge:
            await db.expunge(obj)
        return obj

    def _get_sync_object() -> T:
        obj = db.query(db_class).filter(getattr(db_class, lookup_column) == id).one_or_none()
        if obj is None:
            raise HTTPException(status_code=404, detail="The object does not exist.")  # type: ignore
        if expunge:
            db.expunge(obj)
        return obj

    if isinstance(db, AsyncSession):
        return asyncio.ensure_future(_get_async_object())  # type: ignore
    elif isinstance(db, Session):
        return _get_sync_object()
    else:
        raise HTTPException(status_code=404, detail="Invalid session type. Expected Session or AsyncSession.")  # type: ignore


# TODO: Add testing
@overload
async def create_obj_from_data(
    data: BaseModel,
    model: Type[T],
    db: AsyncSession,
    additional_data: dict = {},
    exclude: dict = {},
) -> T:
    pass


@overload
def create_obj_from_data(
    data: BaseModel, model: Type[T], db: Session, additional_data: dict = {}, exclude: dict = {}
) -> T:
    pass


def create_obj_from_data(
    data: BaseModel, model: Type[T], db: Session | AsyncSession, additional_data={}, exclude={}
) -> T:
    obj_data = data.model_dump(exclude=exclude) | additional_data
    obj = model(**obj_data)

    async def _create_async_obj():
        db.add(obj)
        await db.commit()
        await db.refresh(obj)
        return obj

    def _create_sync_obj():
        db.add(obj)
        db.commit()
        db.refresh(obj)
        return obj

    if isinstance(db, AsyncSession):
        return asyncio.ensure_future(_create_async_obj())  # type: ignore
    elif isinstance(db, Session):
        return _create_sync_obj()
    else:
        raise HTTPException(status_code=404, detail="Invalid session type. Expected Session or AsyncSession.")  # type: ignore


# TODO: Add testing
@overload
async def update_obj_from_data(
    data: BaseModel,
    model: Type[T],
    id: UUID | str,
    db: AsyncSession,
    partial: bool = True,
    ignore_fields: list = [],
    additional_data: dict = {},
    exclude: dict = {},
) -> T:
    pass


@overload
def update_obj_from_data(
    data: BaseModel,
    model: Type[T],
    id: UUID | str,
    db: Session,
    partial: bool = True,
    ignore_fields: list = [],
    additional_data: dict = {},
    exclude: dict = {},
) -> T:
    pass


def update_obj_from_data(
    data: BaseModel,
    model: Type[T],
    id: UUID | str,
    db: Session | AsyncSession,
    partial: bool = True,
    ignore_fields=[],
    additional_data={},
    exclude={},
) -> T:
    def _update_fields(obj: T):
        data_dict = data.model_dump(exclude_unset=partial, exclude=exclude)
        data_dict.update(additional_data)

        for field in data_dict:
            if field not in ignore_fields:
                setattr(obj, field, data_dict[field])

    async def _update_async_obj() -> T:
        obj = await get_object_or_404(model, id, db)
        _update_fields(obj)
        await db.commit()
        await db.refresh(obj)
        return obj

    def _update_sync_obj() -> T:
        obj = get_object_or_404(model, id, db)
        _update_fields(obj)
        db.commit()
        db.refresh(obj)
        return obj

    if isinstance(db, AsyncSession):
        return asyncio.ensure_future(_update_async_obj())  # type: ignore
    elif isinstance(db, Session):
        return _update_sync_obj()
    else:
        raise HTTPException(status_code=404, detail="Invalid session type. Expected Session or AsyncSession.")  # type: ignore


# TODO: Add testing
@overload
async def delete_object(db_class: Type[T], id: UUID | str, db: AsyncSession) -> None:
    pass


@overload
def delete_object(db_class: Type[T], id: UUID | str, db: Session) -> None:
    pass


def delete_object(db_class: Type[T], id: UUID | str, db: Session | AsyncSession) -> None:
    async def _delete_async_obj() -> None:
        query = select(db_class).filter(db_class.id == id)
        result = await db.execute(query)
        obj = result.scalar_one_or_none()
        if obj is None:
            raise HTTPException(status_code=404, detail="The object does not exist.")
        await db.delete(obj)
        await db.commit()

    def _delete_sync_obj() -> None:
        obj = db.query(db_class).filter(db_class.id == id).one_or_none()
        if obj is None:
            raise HTTPException(status_code=404, detail="The object does not exist.")
        db.delete(obj)
        db.commit()

    if isinstance(db, AsyncSession):
        return asyncio.ensure_future(_delete_async_obj())  # type: ignore
    elif isinstance(db, Session):
        return _delete_sync_obj()
    else:
        raise HTTPException(status_code=404, detail="Invalid session type. Expected Session or AsyncSession.")  # type: ignore



from typing import AsyncGenerator

from sqlalchemy.ext.asyncio import AsyncSession, async_sessionmaker, create_async_engine

from .common import SQLALCHEMY_DATABASE_URL, name, ssl_mode

async_engine = create_async_engine(
    SQLALCHEMY_DATABASE_URL + name, pool_pre_ping=True, connect_args={"sslmode": ssl_mode}
)

AsyncSessionLocal = async_sessionmaker(
    bind=async_engine,
    class_=AsyncSession,
    expire_on_commit=False,
    autoflush=False,
    autocommit=False,
)


async def get_async_db() -> AsyncGenerator[AsyncSession, None]:
    async with AsyncSessionLocal() as db:
        yield db



from contextlib import suppress
from math import ceil
from typing import Any, Generic, Optional, Self, Sequence, TypeVar, Union, overload

from fastapi import Query
from fastapi_pagination.api import apply_items_transformer, create_page
from fastapi_pagination.bases import AbstractPage, AbstractParams, RawParams
from fastapi_pagination.ext.sqlalchemy import create_paginate_query
from fastapi_pagination.types import (
    AdditionalData,
    AsyncItemsTransformer,
    GreaterEqualOne,
    GreaterEqualZero,
    ItemsTransformer,
    SyncItemsTransformer,
)
from fastapi_pagination.utils import verify_params
from pydantic import BaseModel
from pydantic.json_schema import SkipJsonSchema
from sqlalchemy import func, select
from sqlalchemy.ext.asyncio import AsyncSession, async_scoped_session
from sqlalchemy.orm.session import Session
from sqlalchemy.sql.selectable import Select
from sqlalchemy.util import await_only, greenlet_spawn

T = TypeVar("T")


class PaginationParams(BaseModel, AbstractParams):
    page: int = Query(1, ge=1, description="Page number")
    size: int = Query(50, ge=1, description="Page size")
    pagination: bool = Query(True, description="Toggle pagination")

    def to_raw_params(self) -> RawParams:
        if not self.pagination:
            return RawParams(limit=None, offset=None)

        return RawParams(limit=self.size, offset=(self.page - 1) * self.size)


# TODO: Add complete fastapi-pagination proxy here
# TODO: Add pagination off functionality
# SkipJsonSchema is used to avoid generating invalid JSON schema in FastAPI
class Page(AbstractPage[T], Generic[T]):
    results: Sequence[T]
    page: GreaterEqualOne | SkipJsonSchema[None] = None
    size: GreaterEqualOne | SkipJsonSchema[None] = None
    pages: GreaterEqualZero | SkipJsonSchema[None] = None
    total: GreaterEqualZero
    has_next: bool | SkipJsonSchema[None] = None
    has_prev: bool | SkipJsonSchema[None] = None

    __params_type__ = PaginationParams

    @classmethod
    def create(
        cls,
        items: Sequence[T],
        params: AbstractParams,
        *,
        total: Optional[int] = None,
        **kwargs: Any,
    ) -> Self:
        if not isinstance(params, PaginationParams):
            raise TypeError("Page should be used with Params")

        size = params.size or total or len(items)
        page = params.page or 1
        pages = None

        if total is not None:
            if total == 0:
                pages = 1
            else:
                pages = ceil(total / size)

        has_next = page < (pages or 1)
        has_prev = page > 1

        return cls(
            total=total,
            results=items,
            page=page,
            size=size,
            pages=pages,
            has_next=has_next,
            has_prev=has_prev,
        )


# Parse response from an SDK to a PAGE
def parse_page(response, page: int, size: int) -> Page:
    return Page(
        page=page,
        size=size,
        total=response.total,
        results=response.results,
        pages=response.pages or 1,
        has_next=response.has_next,
        has_prev=response.has_prev,
    )


def create_count_query(query: Select) -> Select:
    return select(func.count()).select_from(query.subquery())


def unwrap_scalars(
    items: Sequence[Sequence[T]],
    force_unwrap: bool = True,
) -> Union[Sequence[T], Sequence[Sequence[T]]]:
    return [item[0] if force_unwrap else item for item in items]


def _get_sync_conn_from_async(conn: Any) -> Session:  # pragma: no cover
    if isinstance(conn, async_scoped_session):
        conn = conn()

    with suppress(AttributeError):
        return conn.sync_session  # type: ignore

    with suppress(AttributeError):
        return conn.sync_connection  # type: ignore

    raise TypeError("conn must be an AsyncConnection or AsyncSession")


@overload
def paginate(
    connection: Session,
    query: Select,
    params: Optional[AbstractParams] = None,
    transformer: Optional[SyncItemsTransformer] = None,
    additional_data: Optional[AdditionalData] = None,
) -> Any:
    pass


@overload
async def paginate(
    connection: AsyncSession,
    query: Select,
    params: Optional[AbstractParams] = None,
    transformer: Optional[AsyncItemsTransformer] = None,
    additional_data: Optional[AdditionalData] = None,
) -> Any:
    pass


def _paginate(
    connection: Session,
    query: Select,
    params: Optional[AbstractParams] = None,
    transformer: Optional[ItemsTransformer] = None,
    additional_data: Optional[AdditionalData] = None,
    async_: bool = False,
):

    if async_:

        def _apply_items_transformer(*args: Any, **kwargs: Any) -> Any:
            return await_only(apply_items_transformer(*args, **kwargs, async_=True))

    else:
        _apply_items_transformer = apply_items_transformer

    params, raw_params = verify_params(params, "limit-offset", "cursor")
    count_query = create_count_query(query)
    total = connection.scalar(count_query)

    if params.pagination is False and total > 0:
        params = PaginationParams(page=1, size=total)
    else:
        params = PaginationParams(page=params.page, size=params.size)

    query = create_paginate_query(query, params)
    items = connection.execute(query).all()

    items = unwrap_scalars(items)
    t_items = _apply_items_transformer(items, transformer)

    return create_page(
        t_items,
        params=params,
        total=total,
        **(additional_data or {}),
    )


def paginate(
    connection: Session,
    query: Select,
    params: Optional[AbstractParams] = None,
    transformer: Optional[ItemsTransformer] = None,
    additional_data: Optional[AdditionalData] = None,
):
    if isinstance(connection, AsyncSession):
        connection = _get_sync_conn_from_async(connection)
        return greenlet_spawn(
            _paginate, connection, query, params, transformer, additional_data, async_=True
        )

    return _paginate(connection, query, params, transformer, additional_data, async_=False)



import secrets
import string


def create_random_password(length: int = 12) -> str:
    all_characters = string.ascii_letters + string.digits + string.punctuation

    password = [
        secrets.choice(string.ascii_lowercase),
        secrets.choice(string.ascii_uppercase),
        secrets.choice(string.digits),
        secrets.choice(string.punctuation),
    ]
    password += [secrets.choice(all_characters) for _ in range(length - 4)]
    secrets.SystemRandom().shuffle(password)
    return "".join(password)



import uuid
from datetime import datetime

from sqlalchemy import Column, DateTime, String
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import as_declarative
from sqlalchemy.sql import func

from .mixins import AutoAnnotateMixin, AutoInitMixin


@as_declarative()
class Base(AutoAnnotateMixin, AutoInitMixin):
    __abstract__ = True
    # Primary key as uuid
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    created_at = Column(DateTime(timezone=True), server_default=func.now())
    updated_at = Column(DateTime, default=datetime.utcnow, onupdate=datetime.utcnow)
    created_by_id = Column(String)

    __name__: str

    # TODO: Add automated foreign key resolution

    # Generate __tablename__ automatically
    @declared_attr
    def __tablename__(cls) -> str:
        return cls.__name__.lower()

    def __str__(self) -> str:
        # if the object has a name, title or similar attribute, return it
        if hasattr(self, "name"):
            return str(self.name)  # type: ignore

        # if the object has a title attribute, return it
        if hasattr(self, "title"):
            return str(self.title)  # type: ignore

        # otherwise return the object's id
        return str(self.id)



from sqlalchemy import Column
from sqlalchemy.orm import Mapped


class AutoAnnotateMixin:
    @classmethod
    def __init_subclass__(cls) -> None:
        super().__init_subclass__()
        annotations = {}
        for key, value in cls.__dict__.items():
            if isinstance(value, Column):
                annotations[key] = Mapped[value.type.python_type]
        cls.__annotations__ = annotations


class AutoInitMixin:
    @classmethod
    def __init_subclass__(cls) -> None:
        super().__init_subclass__()
        init_params = []
        for key, value in cls.__dict__.items():
            if isinstance(value, Column):
                if not value.nullable and not value.default and not value.server_default:
                    init_params.append((key, value.type.python_type))

        def __init__(self, **kwargs):
            super(cls, self).__init__()
            for key, _ in init_params:
                if key not in kwargs:
                    raise TypeError(f"Missing required argument: {key}")
                setattr(self, key, kwargs[key])
            for key, value in kwargs.items():
                if key not in init_params and hasattr(self.__class__, key):
                    setattr(self, key, value)

        cls.__init__ = __init__



import json

from fastapi.testclient import TestClient


class GenericClient(TestClient):
    def __init__(self, client):
        self.c = TestClient(client)
        self.default_headers = {}

    def get(self, url: str, r_code: int = 200, parse_json=True):
        re = self.c.get(url, headers=self.default_headers)
        if re.status_code != r_code:
            print(re.content)
        assert r_code == re.status_code
        return re.json() if parse_json else re.content

    def delete(self, url: str, r_code: int = 204):
        re = self.c.delete(url, headers=self.default_headers)
        if re.status_code != r_code:
            print(re.content)
        assert r_code == re.status_code
        return re.json() if r_code != 204 else None

    def post(
        self, url: str, obj: dict | str = {}, r_code: int = 201, raw_response=False, *args, **kwargs
    ):
        re = self.c.post(
            url,
            data=json.dumps(obj) if type(obj) == dict else obj,
            headers=self.default_headers | {"Content-Type": "application/json"},
            *args,
            **kwargs,
        )
        if re.status_code != r_code:
            print(re.content)
        assert r_code == re.status_code
        return re.json() if not raw_response else re

    def post_file(self, url: str, file, r_code: int = 201, raw_response=False, *args, **kwargs):
        re = self.c.post(
            url,
            files={"file": file},
            headers=self.default_headers,
            *args,
            **kwargs,
        )
        if re.status_code != r_code:
            print(re.content)
        assert r_code == re.status_code
        return re.json() if not raw_response else re

    def patch(
        self, url: str, obj: dict | str = {}, r_code: int = 200, raw_response=False, *args, **kwargs
    ):
        re = self.c.patch(
            url,
            data=json.dumps(obj) if type(obj) == dict else obj,
            headers=self.default_headers | {"Content-Type": "application/json"},
            *args,
            **kwargs,
        )
        if re.status_code != r_code:
            print(re.content)
        assert r_code == re.status_code
        return re.json() if not raw_response else re

    def put(
        self, url: str, obj: dict | str = {}, r_code: int = 200, raw_response=False, *args, **kwargs
    ):
        re = self.c.put(
            url,
            data=json.dumps(obj) if type(obj) == dict else obj,
            headers=self.default_headers
            | {
                "Content-Type": "application/json",
                "accept": "application/json",
            },
            *args,
            **kwargs,
        )
        if re.status_code != r_code:
            print(re.content)
        assert r_code == re.status_code
        return re.json() if not raw_response else re

    def obj_lifecycle(
        self,
        input_obj: dict,
        url: str,
        pagination: bool = True,
        id_field: str = "id",
        created_at_check: bool = True,
    ):
        # GET LIST
        re = self.get(url)
        if pagination:
            assert re["total"] == 0
            assert len(re["results"]) == 0
        else:
            assert len(re) == 0

        # CREATE
        re = self.post(url, obj=input_obj)
        assert id_field in re
        assert re[id_field] is not None

        if created_at_check:
            assert "created_at" in re
            assert re["created_at"] is not None

        obj_id = str(re[id_field])

        # GET
        re = self.get(f"{url}{obj_id}/")
        assert re[id_field] == obj_id

        # GET LIST
        re = self.get(url)
        if pagination:
            assert re["total"] == 1
            assert len(re["results"]) == 1
        else:
            assert len(re) == 1

        # DELETE
        self.delete(f"{url}{obj_id}")

        # GET LIST
        re = self.get(url)
        if pagination:
            assert re["total"] == 0
            assert len(re["results"]) == 0
        else:
            assert len(re) == 0

        # GET
        self.get(f"{url}{obj_id}", parse_json=False, r_code=404)



import json
import unittest
from typing import Type

from httpx import ASGITransport, AsyncClient, Response
from sqlalchemy import create_engine, text
from sqlalchemy.ext.asyncio import create_async_engine
from sqlalchemy_utils import create_database, database_exists, drop_database

from creyPY.fastapi.models.base import Base


class AbstractTestAPI(unittest.IsolatedAsyncioTestCase):
    client: AsyncClient
    default_headers: dict = {}

    @classmethod
    def setUpClass(cls, app, headers={}) -> None:
        cls.client = AsyncClient(
            transport=ASGITransport(app=app), base_url="http://testserver", follow_redirects=True
        )
        cls.default_headers = headers

    @classmethod
    def setup_database(
        cls,
        sync_db_url: str,
        async_db_url: str,
        base: Type[Base],
        btree_gist: bool = False,
        ssl_mode: str = "require",
    ):
        cls.engine_s = create_engine(
            sync_db_url,
            echo=False,
            pool_pre_ping=True,
            connect_args={"sslmode": ssl_mode},
        )
        if database_exists(cls.engine_s.url):
            drop_database(cls.engine_s.url)
        create_database(cls.engine_s.url)

        if btree_gist:
            with cls.engine_s.begin() as conn:
                conn.execute(text("CREATE EXTENSION IF NOT EXISTS btree_gist"))

        # Migrate
        base.metadata.create_all(cls.engine_s)

        cls.engine = create_async_engine(
            async_db_url,
            echo=False,
            pool_pre_ping=True,
            connect_args={"sslmode": ssl_mode},
        )

    async def get(self, url: str, r_code: int = 200, parse_json=True) -> dict | bytes:
        re = await self.client.get(url, headers=self.default_headers)
        if re.status_code != r_code:
            print(re.content)
        self.assertEqual(r_code, re.status_code)
        return re.json() if parse_json else re.content

    async def delete(self, url: str, r_code: int = 204) -> dict | None:
        re = await self.client.delete(url, headers=self.default_headers)
        if re.status_code != r_code:
            print(re.content)
        self.assertEqual(r_code, re.status_code)
        return re.json() if r_code != 204 else None

    async def post(
        self, url: str, obj: dict | str = {}, r_code: int = 201, raw_response=False, *args, **kwargs
    ):
        re = await self.client.post(
            url,
            data=json.dumps(obj) if isinstance(obj, dict) else obj,
            headers=self.default_headers | {"Content-Type": "application/json"},
            *args,
            **kwargs,
        )
        if re.status_code != r_code:
            print(re.content)
        if not raw_response:
            self.assertEqual(r_code, re.status_code)
        return re.json() if not raw_response else re

    async def post_file(
        self, url: str, file, r_code: int = 201, raw_response=False, *args, **kwargs
    ) -> dict | bytes | Response:
        re = await self.client.post(
            url,
            files={"file": file},
            headers=self.default_headers,
            *args,
            **kwargs,
        )
        if re.status_code != r_code:
            print(re.content)
        self.assertEqual(r_code, re.status_code)
        return re.json() if not raw_response else re

    async def patch(
        self, url: str, obj: dict | str = {}, r_code: int = 200, raw_response=False, *args, **kwargs
    ) -> dict | bytes | Response:
        re = await self.client.patch(
            url,
            data=json.dumps(obj) if isinstance(obj, dict) else obj,
            headers=self.default_headers | {"Content-Type": "application/json"},
            *args,
            **kwargs,
        )
        if re.status_code != r_code:
            print(re.content)
        self.assertEqual(r_code, re.status_code)
        return re.json() if not raw_response else re

    async def put(
        self, url: str, obj: dict | str = {}, r_code: int = 200, raw_response=False, *args, **kwargs
    ) -> dict | bytes | Response:
        re = await self.client.put(
            url,
            data=json.dumps(obj) if isinstance(obj, dict) else obj,
            headers=self.default_headers
            | {
                "Content-Type": "application/json",
                "accept": "application/json",
            },
            *args,
            **kwargs,
        )
        if re.status_code != r_code:
            print(re.content)
        self.assertEqual(r_code, re.status_code)
        return re.json() if not raw_response else re

    async def obj_lifecycle(
        self,
        input_obj: dict,
        url: str,
        pagination: bool = True,
        id_field: str = "id",
        created_at_check: bool = True,
    ):
        # GET LIST
        re = await self.get(url)
        if pagination:
            self.assertEqual(re["total"], 0)
            self.assertEqual(len(re["results"]), 0)
        else:
            self.assertEqual(len(re), 0)

        # CREATE
        re = await self.post(url, obj=input_obj)
        self.assertIn(id_field, re)
        self.assertIsNotNone(re[id_field])

        if created_at_check:
            self.assertIn("created_at", re)
            self.assertIsNotNone(re["created_at"])

        obj_id = str(re[id_field])

        # GET
        re = await self.get(f"{url}{obj_id}/")
        self.assertEqual(re[id_field], obj_id)

        # GET LIST
        re = await self.get(url)
        if pagination:
            self.assertEqual(re["total"], 1)
            self.assertEqual(len(re["results"]), 1)
        else:
            self.assertEqual(len(re), 1)

        # DELETE
        await self.delete(f"{url}{obj_id}")

        # GET LIST
        re = await self.get(url)
        if pagination:
            self.assertEqual(re["total"], 0)
            self.assertEqual(len(re["results"]), 0)
        else:
            self.assertEqual(len(re), 0)

        # GET
        await self.get(f"{url}{obj_id}", parse_json=False, r_code=404)



import requests
from cachetools import TTLCache, cached

from .common import AUTH0_CLIENT_ID, AUTH0_CLIENT_SECRET, AUTH0_DOMAIN

cache = TTLCache(maxsize=100, ttl=600)


@cached(cache)
def get_management_token() -> str:
    response = requests.post(
        f"https://{AUTH0_DOMAIN}/oauth/token",
        json={
            "client_id": AUTH0_CLIENT_ID,
            "client_secret": AUTH0_CLIENT_SECRET,
            "audience": f"https://{AUTH0_DOMAIN}/api/v2/",  # This should be the management audience
            "grant_type": "client_credentials",
        },
        timeout=5,  # Add a timeout parameter to avoid hanging requests
    ).json()
    return response["access_token"]



import enum


# Supported currencies for Stripe https://stripe.com/docs/currencies?presentment-currency=DE
# Commented out are the Zero-decimal currencies, to avoid custom implementation
class CurrencyEnum(str, enum.Enum):
    USD = "United States Dollar"
    AED = "United Arab Emirates Dirham"
    AFN = "Afghan Afghani"
    ALL = "Albanian Lek"
    AMD = "Armenian Dram"
    ANG = "Netherlands Antillean Guilder"
    AOA = "Angolan Kwanza"
    ARS = "Argentine Peso"
    AUD = "Australian Dollar"
    AWG = "Aruban Florin"
    AZN = "Azerbaijani Manat"
    BAM = "Bosnia-Herzegovina Convertible Mark"
    BBD = "Barbadian Dollar"
    BDT = "Bangladeshi Taka"
    BGN = "Bulgarian Lev"
    # BIF = "Burundian Franc"
    BMD = "Bermudan Dollar"
    BND = "Brunei Dollar"
    BOB = "Bolivian Boliviano"
    BRL = "Brazilian Real"
    BSD = "Bahamian Dollar"
    BWP = "Botswanan Pula"
    BYN = "Belarusian Ruble"
    BZD = "Belize Dollar"
    CAD = "Canadian Dollar"
    CDF = "Congolese Franc"
    CHF = "Swiss Franc"
    # CLP = "Chilean Peso"
    CNY = "Chinese Yuan"
    COP = "Colombian Peso"
    CRC = "Costa Rican Colón"
    CVE = "Cape Verdean Escudo"
    CZK = "Czech Republic Koruna"
    # DJF = "Djiboutian Franc"
    DKK = "Danish Krone"
    DOP = "Dominican Peso"
    DZD = "Algerian Dinar"
    EGP = "Egyptian Pound"
    ETB = "Ethiopian Birr"
    EUR = "Euro"
    FJD = "Fijian Dollar"
    FKP = "Falkland Islands Pound"
    GBP = "British Pound Sterling"
    GEL = "Georgian Lari"
    GIP = "Gibraltar Pound"
    GMD = "Gambian Dalasi"
    # GNF = "Guinean Franc"
    GTQ = "Guatemalan Quetzal"
    GYD = "Guyanaese Dollar"
    HKD = "Hong Kong Dollar"
    HNL = "Honduran Lempira"
    HTG = "Haitian Gourde"
    HUF = "Hungarian Forint"
    IDR = "Indonesian Rupiah"
    ILS = "Israeli New Sheqel"
    INR = "Indian Rupee"
    ISK = "Icelandic Króna"
    JMD = "Jamaican Dollar"
    # JPY = "Japanese Yen"
    KES = "Kenyan Shilling"
    KGS = "Kyrgystani Som"
    KHR = "Cambodian Riel"
    # KMF = "Comorian Franc"
    # KRW = "South Korean Won"
    KYD = "Cayman Islands Dollar"
    KZT = "Kazakhstani Tenge"
    LAK = "Laotian Kip"
    LBP = "Lebanese Pound"
    LKR = "Sri Lankan Rupee"
    LRD = "Liberian Dollar"
    LSL = "Lesotho Loti"
    MAD = "Moroccan Dirham"
    MDL = "Moldovan Leu"
    # MGA = "Malagasy Ariary"
    MKD = "Macedonian Denar"
    MMK = "Myanma Kyat"
    MNT = "Mongolian Tugrik"
    MOP = "Macanese Pataca"
    MUR = "Mauritian Rupee"
    MVR = "Maldivian Rufiyaa"
    MWK = "Malawian Kwacha"
    MXN = "Mexican Peso"
    MYR = "Malaysian Ringgit"
    MZN = "Mozambican Metical"
    NAD = "Namibian Dollar"
    NGN = "Nigerian Naira"
    NIO = "Nicaraguan Córdoba"
    NOK = "Norwegian Krone"
    NPR = "Nepalese Rupee"
    NZD = "New Zealand Dollar"
    PAB = "Panamanian Balboa"
    PEN = "Peruvian Nuevo Sol"
    PGK = "Papua New Guinean Kina"
    PHP = "Philippine Peso"
    PKR = "Pakistani Rupee"
    PLN = "Polish Zloty"
    # PYG = "Paraguayan Guarani"
    QAR = "Qatari Rial"
    RON = "Romanian Leu"
    RSD = "Serbian Dinar"
    RUB = "Russian Ruble"
    # RWF = "Rwandan Franc"
    SAR = "Saudi Riyal"
    SBD = "Solomon Islands Dollar"
    SCR = "Seychellois Rupee"
    SEK = "Swedish Krona"
    SGD = "Singapore Dollar"
    SHP = "Saint Helena Pound"
    SLE = "Sierra Leonean Leone"
    SOS = "Somali Shilling"
    SRD = "Surinamese Dollar"
    STD = "São Tomé and Príncipe Dobra"
    SZL = "Swazi Lilangeni"
    THB = "Thai Baht"
    TJS = "Tajikistani Somoni"
    TOP = "Tongan Pa`anga"
    TRY = "Turkish Lira"
    TTD = "Trinidad and Tobago Dollar"
    TWD = "New Taiwan Dollar"
    TZS = "Tanzanian Shilling"
    UAH = "Ukrainian Hryvnia"
    # UGX = "Ugandan Shilling"
    UYU = "Uruguayan Peso"
    UZS = "Uzbekistan Som"
    # VND = "Vietnamese Dong"
    # VUV = "Vanuatu Vatu"
    WST = "Samoan Tala"
    # XAF = "CFA Franc BEAC"
    XCD = "East Caribbean Dollar"
    # XOF = "CFA Franc BCEAO"
    # XPF = "CFP Franc"
    YER = "Yemeni Rial"
    ZAR = "South African Rand"
    ZMW = "Zambian Kwacha"


class StripeStatus(str, enum.Enum):
    active = "active"
    all = "all"
    canceled = "canceled"
    ended = "ended"
    incomplete = "incomplete"
    incomplete_expired = "incomplete_expired"
    past_due = "past_due"
    paused = "paused"
    trialing = "trialing"
    unpaid = "unpaid"
